#ifndef _UAVC_H_
#define _UAVC_H_




// ============================================================================
//
// USB Video Class 1.1
//
// ============================================================================
#define VIDEO_REQUEST_CLASS_RC_UNDEFINED                    0x00
#define VIDEO_REQUEST_CLASS_SET_CUR                         0x01
#define VIDEO_REQUEST_CLASS_GET_CUR                         0x81
#define VIDEO_REQUEST_CLASS_GET_MIN                         0x82
#define VIDEO_REQUEST_CLASS_GET_MAX                         0x83
#define VIDEO_REQUEST_CLASS_GET_RES                         0x84
#define VIDEO_REQUEST_CLASS_GET_LEN                         0x85
#define VIDEO_REQUEST_CLASS_GET_INFO                        0x86
#define VIDEO_REQUEST_CLASS_GET_DEF                         0x87


// A.9.1 Video Control Interface Control Selectors
#define CONTROL_SELECTOR_VC_CONTROL_UNDEFINED               0x00
#define CONTROL_SELECTOR_VC_VIDEO_POWER_MODE_CONTROL        0x01
#define CONTROL_SELECTOR_VC_REQUEST_ERROR_CODE_CONTROL      0x02

#define VC_REQUEST_ERROR_CODE_NO_ERROR                      0x00
#define VC_REQUEST_ERROR_CODE_NOT_READY                     0x01
#define VC_REQUEST_ERROR_CODE_WRONG_STATE                   0x02
#define VC_REQUEST_ERROR_CODE_POWER                         0x03
#define VC_REQUEST_ERROR_CODE_OUT_OF_RANGE                  0x04
#define VC_REQUEST_ERROR_CODE_INVALID_UNIT                  0x05
#define VC_REQUEST_ERROR_CODE_INVALID_CONTROL               0x06
#define VC_REQUEST_ERROR_CODE_INVALID_REQUEST               0x07
#define VC_REQUEST_ERROR_CODE_UNKNOWN                       0xFF


// A.9.6 Extension Unit Control Selectors
#define XU_CONTROL_UNDEFINED                                0x00
#define XU_CONTROL_VENDOR_REQUEST                           0x01
#define XU_CONTROL_TRANSFER_BUF1                            0x02
#define XU_CONTROL_TRANSFER_BUF2                            0x03

// A.9.7 Video Streaming Interface Control Selectors
#define CONTROL_SELECTOR_VS_CONTROL_UNDEFINED               0x00
#define CONTROL_SELECTOR_VS_PROBE_CONTROL                   0x01
#define CONTROL_SELECTOR_VS_COMMIT_CONTROL                  0x02
#define CONTROL_SELECTOR_VS_STILL_PROBE_CONTROL             0x03
#define CONTROL_SELECTOR_VS_STILL_COMMIT_CONTROL            0x04
#define CONTROL_SELECTOR_VS_STILL_IMAGE_TRIGGER_CONTROL     0x05
#define CONTROL_SELECTOR_VS_STREAM_ERROR_CODE_CONTROL       0x06
#define CONTROL_SELECTOR_VS_GENERATE_KEY_FRAME_CONTROL      0x07
#define CONTROL_SELECTOR_VS_UPDATE_FRAME_SEGMENT_CONTROL    0x08
#define CONTROL_SELECTOR_VS_SYNCH_DELAY_CONTROL             0x09




#define INTERFACE_CLASS_CC_VIDEO                            0x0E


#define INTERFACE_SUBCLASS_SC_UNDEFINED                     0x00
#define INTERFACE_SUBCLASS_SC_VIDEOCONTROL                  0x01
#define INTERFACE_SUBCLASS_SC_VIDEOSTREAMING                0x02
#define INTERFACE_SUBCLASS_SC_VIDEO_INTERFACE_COLLECTION    0x03


#define INTERFACE_PROTOCOL_PC_PROTOCOL_UNDEFINED            0x00




#define DESCRIPTOR_TYPE_CS_UNDEFINED                        0x20
#define DESCRIPTOR_TYPE_CS_DEVICE                           0x21
#define DESCRIPTOR_TYPE_CS_CONFIGURATION                    0x22
#define DESCRIPTOR_TYPE_CS_STRING                           0x23
#define DESCRIPTOR_TYPE_CS_INTERFACE                        0x24
#define DESCRIPTOR_TYPE_CS_ENDPOINT                         0x25


#define DESCRIPTOR_SUBTYPE_VC_DESCRIPTOR_UNDEFINED          0x00
#define DESCRIPTOR_SUBTYPE_VC_HEADER                        0x01
#define DESCRIPTOR_SUBTYPE_VC_INPUT_TERMINAL                0x02
#define DESCRIPTOR_SUBTYPE_VC_OUTPUT_TERMINAL               0x03
#define DESCRIPTOR_SUBTYPE_VC_SELECTOR_UNIT                 0x04
#define DESCRIPTOR_SUBTYPE_VC_PROCESSING_UNIT               0x05
#define DESCRIPTOR_SUBTYPE_VC_EXTENSION_UNIT                0x06

#define DESCRIPTOR_SUBTYPE_VS_UNDEFINED                     0x00
#define DESCRIPTOR_SUBTYPE_VS_INPUT_HEADER                  0x01
#define DESCRIPTOR_SUBTYPE_VS_OUTPUT_HEADER                 0x02
#define DESCRIPTOR_SUBTYPE_VS_STILL_IMAGE_FRAME             0x03
#define DESCRIPTOR_SUBTYPE_VS_FORMAT_UNCOMPRESSED           0x04
#define DESCRIPTOR_SUBTYPE_VS_FRAME_UNCOMPRESSED            0x05
#define DESCRIPTOR_SUBTYPE_VS_FORMAT_MJPEG                  0x06
#define DESCRIPTOR_SUBTYPE_VS_FRAME_MJPEG                   0x07
#define DESCRIPTOR_SUBTYPE_VS_FORMAT_MPEG2TS                0x0A
#define DESCRIPTOR_SUBTYPE_VS_FORMAT_DV                     0x0C
#define DESCRIPTOR_SUBTYPE_VS_COLORFORMAT                   0x0D
#define DESCRIPTOR_SUBTYPE_VS_FORMAT_FRAME_BASED            0x10
#define DESCRIPTOR_SUBTYPE_VS_FRAME_FRAME_BASED             0x11
#define DESCRIPTOR_SUBTYPE_VS_FORMAT_STREAM_BASED           0x12

#define DESCRIPTOR_SUBTYPE_EP_UNDEFINED                     0x00
#define DESCRIPTOR_SUBTYPE_EP_GENERAL                       0x01
#define DESCRIPTOR_SUBTYPE_EP_ENDPOINT                      0x02
#define DESCRIPTOR_SUBTYPE_EP_INTERRUPT                     0x03




typedef struct _VC_INTERFACE_HEADER_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bDescriptorSubType;
    U16 bcdUVC;
    U16 wTotalLength;
    U32 dwClockFrequency;
    U8  bInCollection;
    U8  baInterfaceNr[1];
} VC_INTERFACE_HEADER_DESCRIPTOR, *PVC_INTERFACE_HEADER_DESCRIPTOR;




#define TERMINAL_TYPE_TT_VENDOR_SPECIFIC                0x0100
#define TERMINAL_TYPE_TT_STREAMING                      0x0101

#define TERMINAL_TYPE_ITT_VENDOR_SPECIFIC               0x0200
#define TERMINAL_TYPE_ITT_CAMERA                        0x0201
#define TERMINAL_TYPE_ITT_MEDIA_TRANSPORT_INPUT         0x0202

#define TERMINAL_TYPE_OTT_VENDOR_SPECIFIC               0x0300
#define TERMINAL_TYPE_OTT_DISPLAY                       0x0301
#define TERMINAL_TYPE_OTT_MEDIA_TRANSPORT_OUTPUT        0x0302

#define TERMINAL_TYPE_EXTERNAL_VENDOR_SPECIFIC          0x0400
#define TERMINAL_TYPE_COMPOSITE_CONNECTOR               0x0401
#define TERMINAL_TYPE_SVIDEO_CONNECTOR                  0x0402
#define TERMINAL_TYPE_COMPONENT_CONNECTOR               0x0403




typedef struct _OUTPUT_TERMINAL_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bDescriptorSubtype;
    U8  bTerminalID;
    U16 wTerminalType;
    U8  bAssocTerminal;
    U8  bSourceID;
    U8  iTerminal;
} OUTPUT_TERMINAL_DESCRIPTOR, *POUTPUT_TERMINAL_DESCRIPTOR;




typedef struct _EXTENSION_UNIT_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bDescriptorSubtype;
    U8  bUnitID;
    U8  guidExtensionCode[16];
    U8  bNumControls;
    U8  bNrInPins;
    U8  baSourceID;
    U8  bControlSize;
    U8  bmControls[1];
    U8  iExtension;
} EXTENSION_UNIT_DESCRIPTOR, *PEXTENSION_UNIT_DESCRIPTOR;




typedef struct _CAMERA_TERMINAL_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bDescriptorSubtype;
    U8  bTerminalID;
    U16 wTerminalType;
    U8  bAssocTerminal;
    U8  iTerminal;
    U16 wObjectiveFocalLengthMin;
    U16 wObjectiveFocalLengthMax;
    U16 wOcularFocalLength;
    U8  bControlSize;
    U8  bmControls[3];
} CAMERA_TERMINAL_DESCRIPTOR, *PCAMERA_TERMINAL_DESCRIPTOR;




typedef struct _PROCESSING_UNIT_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bDescriptorSubtype;
    U8  bUnitID;
    U8  bSourceID;
    U16 wMaxMultiplier;
    U8  bControlSize;
    U8  bmControls[2];
    U8  iProcessing;
} PROCESSING_UNIT_DESCRIPTOR, *PPROCESSING_UNIT_DESCRIPTOR;




typedef struct _VS_INTERFACE_INPUT_HEADER_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bDescriptorSubtype;
    U8  bNumFormats;
    U16 wTotalLength;
    U8  bEndpointAddress;
    U8  bmInfo;
    U8  bTerminalLink;
    U8  bStillCaptureMethod;
    U8  bTriggerSupport;
    U8  bTriggerUsage;
    U8  bControlSize;
    U8  bmaControls[2];                         // Support two formats
} VS_INTERFACE_INPUT_HEADER_DESCRIPTOR, *PVS_INTERFACE_INPUT_HEADER_DESCRIPTOR;




typedef struct _COLOR_MATCHING_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bDescriptorSubtype;
    U8  bColorPrimaries;
    U8  bTransferCharacteristics;
    U8  bMatrixCoefficients;
} COLOR_MATCHING_DESCRIPTOR, *PCOLOR_MATCHING_DESCRIPTOR;

#define COLOR_PRIMARIES_UNSPECIFIED                 0x00
#define COLOR_PRIMARIES_BT_709_SRGB                 0x01        // Default
#define COLOR_PRIMARIES_BT_470_2_M                  0x02
#define COLOR_PRIMARIES_BT_470_2_BG                 0x03
#define COLOR_PRIMARIES_SMPTE_170M                  0x04
#define COLOR_PRIMARIES_SMPTE_240M                  0x05

#define TRANSFER_CHARACTERISTICS_UNSPECIFIED        0x00
#define TRANSFER_CHARACTERISTICS_BT_709             0x01        // Default
#define TRANSFER_CHARACTERISTICS_BT_470_2_M         0x02
#define TRANSFER_CHARACTERISTICS_BT_470_2_BG        0x03
#define TRANSFER_CHARACTERISTICS_SMPTE_170M         0x04
#define TRANSFER_CHARACTERISTICS_SMPTE_240M         0x05
#define TRANSFER_CHARACTERISTICS_LINEAR             0x06
#define TRANSFER_CHARACTERISTICS_SRGB               0x07

#define MATRIX_COEFFICIENTS_UNSPECIFIED             0x00
#define MATRIX_COEFFICIENTS_BT_709                  0x01
#define MATRIX_COEFFICIENTS_FCC                     0x02
#define MATRIX_COEFFICIENTS_BT_470_2_BG             0x03
#define MATRIX_COEFFICIENTS_SMPTE_170M              0x04        // Default
#define MATRIX_COEFFICIENTS_SMPTE_240M              0x05


// ============================================================================
//
//STREAMING_PARAMETER USB Video Class v1.0
//
// ============================================================================

typedef struct _STREAMING_PARAMETER
{
    U16 bmHint;
    U8  bFormatIndex;
    U8  bFrameIndex;
    U32 dwFrameInterval;
    U16 wKeyFrameRate;
    U16 wPFrameRate;
    U16 wCompQuality;
    U16 wCompWindowSize;
    U16 wDelay;
    U32 dwMaxVideoFrameSize;
    U32 dwMaxPayloadTransferSize;
} STREAMING_PARAMETER, *PSTREAMING_PARAMETER;




// ============================================================================
//
// USB Video Payload Uncompressed 1.0
//
// ============================================================================
typedef struct _UNCOMPRESSED_VIDEO_FORMAT_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bDescriptorSubtype;
    U8  bFormatIndex;
    U8  bNumFrameDescriptors;
    U8  guidFormat[16];
    U8  bBitsPerPixel;
    U8  bDefaultFrameIndex;
    U8  bAspectRatioX;
    U8  bAspectRatioY;
    U8  bmInterlaceFlags;
    U8  bCopyProtect;
} UNCOMPRESSED_VIDEO_FORMAT_DESCRIPTOR, *PUNCOMPRESSED_VIDEO_FORMAT_DESCRIPTOR;


typedef struct _UNCOMPRESSED_VIDEO_FRAME_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bDescriptorSubtype;
    U8  bFrameIndex;
    U8  bmCapabilities;
    U16 wWidth;
    U16 wHeight;
    U32 dwMinBitRate;
    U32 dwMaxBitRate;
    U32 dwMaxVideoFrameBufferSize;
    U32 dwDefaultFrameInterval;
    U8  bFrameIntervalType;
    U32 dwFrameInterval[1];                             // Support one Frame Interval 30 fps
} UNCOMPRESSED_VIDEO_FRAME_DESCRIPTOR, *PUNCOMPRESSED_VIDEO_FRAME_DESCRIPTOR;




// ============================================================================
//
// USB Video Payload MJPEG 1.0
//
// ============================================================================
typedef struct _MOTION_JPEG_VIDEO_FORMAT_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bDescriptorSubtype;
    U8  bFormatIndex;
    U8  bNumFrameDescriptors;
    U8  bmFlags;
    U8  bDefaultFrameIndex;
    U8  bAspectRatioX;
    U8  bAspectRatioY;
    U8  bmInterlaceFlags;
    U8  bCopyProtect;
} MOTION_JPEG_VIDEO_FORMAT_DESCRIPTOR, *PMOTION_JPEG_VIDEO_FORMAT_DESCRIPTOR;


typedef struct _MOTION_JPEG_VIDEO_FRAME_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bDescriptorSubtype;
    U8  bFrameIndex;
    U8  bmCapabilities;
    U16 wWidth;
    U16 wHeight;
    U32 dwMinBitRate;
    U32 dwMaxBitRate;
    U32 dwMaxVideoFrameBufferSize;
    U32 dwDefaultFrameInterval;
    U8  bFrameIntervalType;
    U32 dwFrameInterval[1];                             // Support one Frame Interval 30 fps
} MOTION_JPEG_VIDEO_FRAME_DESCRIPTOR, *PMOTION_JPEG_VIDEO_FRAME_DESCRIPTOR;




// ============================================================================
//
// USB Audio Class 1.0
//
// ============================================================================
#define AUDIO_REQUEST_CLASS_RC_UNDEFINED                0x00
#define AUDIO_REQUEST_CLASS_SET_CUR                     0x01
#define AUDIO_REQUEST_CLASS_GET_CUR                     0x81
#define AUDIO_REQUEST_CLASS_GET_MIN                     0x82
#define AUDIO_REQUEST_CLASS_GET_MAX                     0x83
#define AUDIO_REQUEST_CLASS_GET_RES                     0x84
#define AUDIO_REQUEST_CLASS_GET_MEM                     0x85




#define CONTROL_SELECTOR_AS_CONTROL_UNDEFINED           0x00
#define CONTROL_SELECTOR_AS_MUTE_CONTROL                0x01
#define CONTROL_SELECTOR_AS_VOLUME_CONTROL              0x02
#define CONTROL_SELECTOR_AS_BASS_CONTROL                0x03
#define CONTROL_SELECTOR_AS_MID_CONTROL                 0x04
#define CONTROL_SELECTOR_AS_TREBLE_CONTROL              0x05
#define CONTROL_SELECTOR_AS_GRAPHIC_EQUALIZER_CONTROL   0x06
#define CONTROL_SELECTOR_AS_AUTOMATIC_GAIN_CONTROL      0x07
#define CONTROL_SELECTOR_AS_DELAY_CONTROL               0x08
#define CONTROL_SELECTOR_AS_BASS_BOOST_CONTROL          0x09
#define CONTROL_SELECTOR_AS_LOUDNESS_CONTROL            0x0A




// Audio Spec. 1.0 - A.1
#define INTERFACE_CLASS_CC_AUDIO                        0x01


// Audio Spec. 1.0 - A.2
#define INTERFACE_SUBCLASS_SC_UNDEFINED                 0x00
#define INTERFACE_SUBCLASS_SC_AUDIOCONTROL              0x01
#define INTERFACE_SUBCLASS_SC_AUDIOSTREAMING            0x02
#define INTERFACE_SUBCLASS_SC_MIDISTREAMING             0x03


// Audio Spec. 1.0 - A.3
#define INTERFACE_PROTOCOL_PR_PROTOCOL_UNDEFINED        0x00




// Audio Spec. 1.0 - A.5
#define DESCRIPTOR_SUBTYPE_AC_DESCRIPTOR_UNDEFINED      0x00
#define DESCRIPTOR_SUBTYPE_AC_HEADER                    0x01
#define DESCRIPTOR_SUBTYPE_AC_INPUT_TERMINAL            0x02
#define DESCRIPTOR_SUBTYPE_AC_OUTPUT_TERMINAL           0x03
#define DESCRIPTOR_SUBTYPE_AC_MIXER_UNIT                0x04
#define DESCRIPTOR_SUBTYPE_AC_SELECTOR_UNIT             0x05
#define DESCRIPTOR_SUBTYPE_AC_FEATURE_UNIT              0x06
#define DESCRIPTOR_SUBTYPE_AC_PROCESSING_UNIT           0x07
#define DESCRIPTOR_SUBTYPE_AC_EXTENSION_UNIT            0x08


// Audio Spec. 1.0 - A.6
#define DESCRIPTOR_SUBTYPE_AS_UNDEFINED                 0x00
#define DESCRIPTOR_SUBTYPE_AS_GENERAL                   0x01
#define DESCRIPTOR_SUBTYPE_AS_FORMAT_TYPE               0x02
#define DESCRIPTOR_SUBTYPE_AS_FORMAT_SPECIFIC           0x03


// Audio Spec. 1.0 - A.7
#define PROCESS_UNIT_TYPE_UNDEFINED                     0x00
#define PROCESS_UNIT_TYPE_UP_DOWNMIX                    0x01
#define PROCESS_UNIT_TYPE_DOLBY_PROLOGIC                0x02
#define PROCESS_UNIT_TYPE_3D_STEREO_EXTENDER            0x03
#define PROCESS_UNIT_TYPE_REVERBERATION                 0x04
#define PROCESS_UNIT_TYPE_CHORUS                        0x05
#define PROCESS_UNIT_TYPE_DYN_RANGE_COMP                0x06


// Audio Spec. 1.0 - A.8
#define EP_CONTROL_SELECTOR_EP_CONTROL_UNDEFINED        0x00
#define EP_CONTROL_SELECTOR_SAMPLING_FREQ_CONTROL       0x01
#define EP_CONTROL_SELECTOR_PITCH_CONTROL               0x02




typedef struct _AC_INTERFACE_HEADER_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bDescriptorSubType;
    U16 bcdADC;
    U16 wTotalLength;
    U8  bInCollection;
    U8  baInterfaceNr[1];
} AC_INTERFACE_HEADER_DESCRIPTOR, *PAC_INTERFACE_HEADER_DESCRIPTOR;




#define TERMINAL_TYPE_ITT_UNDEFINEED                    0x0200
#define TERMINAL_TYPE_ITT_MICROPHONE                    0x0201
#define TERMINAL_TYPE_ITT_DESKTOP_MICROPHONE            0x0202
#define TERMINAL_TYPE_ITT_PERSONAL_MICROPHONE           0x0202

#define TERMINAL_TYPE_OTT_UNDEFINEED                    0x0300
#define TERMINAL_TYPE_OTT_SPEAKER                       0x0301
#define TERMINAL_TYPE_OTT_HEADPHONES                    0x0302




typedef struct _AUDIO_OUTPUT_TERMINAL_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bDescriptorSubtype;
    U8  bTerminalID;
    U16 wTerminalType;
    U8  bAssocTerminal;
    U8  bSourceID;
    U8  iTerminal;
} AUDIO_OUTPUT_TERMINAL_DESCRIPTOR, *PAUDIO_OUTPUT_TERMINAL_DESCRIPTOR;


typedef struct _MICROPHONE_TERMINAL_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bDescriptorSubtype;
    U8  bTerminalID;
    U16 wTerminalType;
    U8  bAssocTerminal;
    U8  bNrChannels;
    U16 wChannelConfig;
    U8  iChannelNames;
    U8  iTerminal;
} MICROPHONE_TERMINAL_DESCRIPTOR, *PMICROPHONE_TERMINAL_DESCRIPTOR;


#define AUDIO_NUMBER_OF_CHANNEL_MONO            1
#define AUDIO_NUMBER_OF_CHANNEL_STEREO          2




typedef struct _FEATURE_UNIT_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bDescriptorSubtype;
    U8  bUnitID;
    U8  bSourceID;
    U8  bControlSize;
    U8  bmaControls[1];
    U8  iFeature;
} FEATURE_UNIT_DESCRIPTOR, *PFEATURE_UNIT_DESCRIPTOR;




typedef struct _AS_INTERFACE_GENERAL_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bDescriptorSubtype;
    U8  bTerminalLink;
    U8  bDelay;
    U16 wFormatTag;
} AS_INTERFACE_GENERAL_DESCRIPTOR, *PAS_INTERFACE_GENERAL_DESCRIPTOR;




typedef struct _STANDARD_AS_ENDPOINT_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bEndpointAddress;
    U8  bmAttributes;
    U16 wMaxPacketSize;
    U8  bInterval;
    U8  bRefresh;
    U8  bSynchAddress;
} STANDARD_AS_ENDPOINT_DESCRIPTOR, *PSTANDARD_AS_ENDPOINT_DESCRIPTOR;




typedef struct _AS_ENDPOINT_GENERAL_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bDescriptorSubtype;
    U8  bmAttributes;
    U8  bLockDelayUnits;
    U16 wLockDelay;
} AS_ENDPOINT_GENERAL_DESCRIPTOR, *PAS_ENDPOINT_GENERAL_DESCRIPTOR;




typedef struct _SAMPLING_FREQ_PARAMETER
{
    U16 SampFreq;
} SAMPLING_FREQ_PARAMETER, *PSAMPLING_FREQ_PARAMETER;




// ============================================================================
//
// USB Audio Data Format 1.0
//
// ============================================================================
typedef struct _AS_INTERFACE_FORMAT_TYPE_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bDescriptorSubtype;
    U8  bFormatType;
    U8  bNrChannels;
    U8  bSubframeSize;
    U8  bBitResolution;
    U8  bSamFreqType;
    U8  tSamFreq[6];                                            // Support two SamFreq
} AS_INTERFACE_FORMAT_TYPE_DESCRIPTOR, *PAS_INTERFACE_FORMAT_TYPE_DESCRIPTOR;

#define AUDIO_FORMAT_TYPE_UNDEFINED                 0x00
#define AUDIO_FORMAT_TYPE_I                         0x01        // Default
#define AUDIO_FORMAT_TYPE_II                        0x02
#define AUDIO_FORMAT_TYPE_III                       0x03

#define AUDIO_DATA_FORMAT_TYPE_I_UNDEFINED          0x00
#define AUDIO_DATA_FORMAT_TYPE_I_PCM                0x01        // Default
#define AUDIO_DATA_FORMAT_TYPE_I_PCM8               0x02
#define AUDIO_DATA_FORMAT_TYPE_I_IEEE_FLOAT         0x03
#define AUDIO_DATA_FORMAT_TYPE_I_ALAW               0x04
#define AUDIO_DATA_FORMAT_TYPE_I_MULAW              0x05




#endif
