#ifndef _USB_H_
#define _USB_H_




typedef struct _SETUP_PACKET
{
    U8  bmRequestType;
    U8  bRequest;
    U16 wValue;
    U16 wIndex;
    U16 wLength;
} SETUP_PACKET, *PSETUP_PACKET;

#define REQUEST_TYPE_DIRECTION                      7:7
#define REQUEST_TYPE_DIRECTION_TO_DEVICE                    0
#define REQUEST_TYPE_DIRECTION_TO_HOST                      1
#define REQUEST_TYPE_TYPE                           6:5
#define REQUEST_TYPE_TYPE_STANDARD                          0
#define REQUEST_TYPE_TYPE_CLASS                             1
#define REQUEST_TYPE_TYPE_VENDOR                            2
#define REQUEST_TYPE_RECIPIENT                      4:0
#define REQUEST_TYPE_RECIPIENT_DEVICE                       0
#define REQUEST_TYPE_RECIPIENT_INTERFACE                    1
#define REQUEST_TYPE_RECIPIENT_ENDPOINT                     2
#define REQUEST_TYPE_RECIPIENT_OTHER                        3

#define REQUEST_STANDARD_GET_STATUS                 0
#define REQUEST_STANDARD_CLEAR_FEATURE              1
#define REQUEST_STANDARD_SET_FEATURE                3
#define REQUEST_STANDARD_SET_ADDRESS                5
#define REQUEST_STANDARD_GET_DESCRIPTOR             6
#define REQUEST_STANDARD_SET_DESCRIPTOR             7
#define REQUEST_STANDARD_GET_CONFIGURATION          8
#define REQUEST_STANDARD_SET_CONFIGURATION          9
#define REQUEST_STANDARD_GET_INTERFACE              10
#define REQUEST_STANDARD_SET_INTERFACE              11
#define REQUEST_STANDARD_SYNCH_FRAME                12
#define REQUEST_STANDARD_SET_SEL                    48
#define REQUEST_STANDARD_ISOCH_DELAY                49




#define DESCRIPTOR_TYPE_DEVICE                      1
#define DESCRIPTOR_TYPE_CONFIGURATION               2
#define DESCRIPTOR_TYPE_STRING                      3
#define DESCRIPTOR_TYPE_INTERFACE                   4
#define DESCRIPTOR_TYPE_ENDPOINT                    5
#define DESCRIPTOR_TYPE_DEVICE_QUALIFIER            6
#define DESCRIPTOR_TYPE_OTHER_SPEED_CONFIG          7
#define DESCRIPTOR_TYPE_INTERFACE_ASSOCIATION       11
#define DESCRIPTOR_TYPE_BOS                         15
#define DESCRIPTOR_TYPE_DEVICE_CAPABILITY           16
#define DESCRIPTOR_TYPE_ENDPOINT_COMPANION          48

#define ENDPOINT_NUMBER_VIDEO_ISO_IN                0x89

#define FEATURE_ENDPOINT_HALT                       0x0000
#define FEATURE_FUNCTION_SUSPEND                    0x0000
#define FEATURE_REMOTE_WAKEUP                       0x0001
#define FEATURE_TEST_MODE                           0x0002
#define FEATURE_U1_ENABLE                           0x0030
#define FEATURE_U2_ENABLE                           0x0031
#define FEATURE_LTM_ENABLE                          0x0032


#define TEST_MODE_RESERVED                          0x00
#define TEST_MODE_TEST_J                            0x01
#define TEST_MODE_TEST_K                            0x02
#define TEST_MODE_TEST_SE0_NAK                      0x03
#define TEST_MODE_TEST_PACKET                       0x04
#define TEST_MODE_TEST_FORCE_ENABLE                 0x05


typedef struct _DEVICE_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U16 bcdUSB;
    U8  bDeviceClass;
    U8  bDeviceSubClass;
    U8  bDeviceProtocol;
    U8  bMaxPacketSize0;
    U16 idVendor;
    U16 idProduct;
    U16 bcdDevice;
    U8  iManufacturer;
    U8  iProduct;
    U8  iSerialNumber;
    U8  bNumConfigurations;
} DEVICE_DESCRIPTOR, *PDEVICE_DESCRIPTOR;


typedef struct _DEVICE_QUALIFIER_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U16 bcdUSB;
    U8  bDeviceClass;
    U8  bDeviceSubClass;
    U8  bDeviceProtocol;
    U8  bMaxPacketSize0;
    U8  bNumConfigurations;
    U8  bReserved;
} DEVICE_QUALIFIER_DESCRIPTOR, *PDEVICE_QUALIFIER_DESCRIPTOR;


typedef struct _CONFIGURATION_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U16 wTotalLength;
    U8  bNumInterfaces;
    U8  bConfigurationValue;
    U8  iConfiguration;
    U8  bmAttributes;
    U8  bMaxPower;
} CONFIGURATION_DESCRIPTOR, *PCONFIGURATION_DESCRIPTOR;


typedef struct _INTERFACE_ASSOCIATION_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bFirstInterface;
    U8  bInterfaceCount;
    U8  bFunctionClass;
    U8  bFunctionSubClass;
    U8  bFunctionProtocol;
    U8  iFunction;
} INTERFACE_ASSOCIATION_DESCRIPTOR, *PINTERFACE_ASSOCIATION_DESCRIPTOR;


typedef struct _INTERFACE_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bInterfaceNumber;
    U8  bAlternateSetting;
    U8  bNumEndpoints;
    U8  bInterfaceClass;
    U8  bInterfaceSubClass;
    U8  bInterfaceProtocol;
    U8  iInterface;
} INTERFACE_DESCRIPTOR, *PINTERFACE_DESCRIPTOR;


typedef struct _ENDPOINT_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bEndpointAddress;
    U8  bmAttributes;
    U16 wMaxPacketSize;
    U8  bInterval;
} ENDPOINT_DESCRIPTOR, *PENDPOINT_DESCRIPTOR;


typedef struct _ENDPOINT_COMPANION_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bMaxBurst;
    U8  bmAttributes;
    U16 wBytesPerInterval;
} ENDPOINT_COMPANION_DESCRIPTOR, *PENDPOINT_COMPANION_DESCRIPTOR;


typedef struct _STRING_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
} STRING_DESCRIPTOR, *PSTRING_DESCRIPTOR;




typedef struct _BOS_DESCRIPTOR
{
    U8  bLength;
    U8  bDescriptorType;
    U16 wTotalLength;
    U8  bNumDeviceCaps;
} BOS_DESCRIPTOR, *PBOS_DESCRIPTOR;


#define DEV_CAPABILITY_TYPE_WIRELESS_USB        1
#define DEV_CAPABILITY_TYPE_USB2_EXTENSION      2
#define DEV_CAPABILITY_TYPE_SUPER_SPEED         3
#define DEV_CAPABILITY_TYPE_CONTAINER_ID        4


typedef struct _DEVICE_CAPABILITY_DESCRIPTOR_USB2_EXTENSION
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bDevCapabilityType;
    U32 bmAttributes;
} DEVICE_CAPABILITY_DESCRIPTOR_USB2_EXTENSION, *PDEVICE_CAPABILITY_DESCRIPTOR_USB2_EXTENSION;


typedef struct _DEVICE_CAPABILITY_DESCRIPTOR_SUPER_SPEED
{
    U8  bLength;
    U8  bDescriptorType;
    U8  bDevCapabilityType;
    U8  bmAttributes;
    U16 wSpeedsSupported;
    U8  bFunctionalitySupport;
    U8  bU1DevExitLat;
    U16 bU2DevExitLat;
} DEVICE_CAPABILITY_DESCRIPTOR_SUPER_SPEED, *PDEVICE_CAPABILITY_DESCRIPTOR_SUPER_SPEED;


#define DEV_CAPABILITY_U1_EXIT_LATENCY          0x4
#define DEV_CAPABILITY_U2_EXIT_LATENCY          0x0004




#endif
